package com.usky.oa.service.impl;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONObject;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.usky.common.core.bean.CommonPage;
import com.usky.common.core.exception.BusinessException;
import com.usky.common.security.utils.SecurityUtils;
import com.usky.oa.domain.*;
import com.usky.oa.mapper.*;
import com.usky.oa.service.OaDocumentService;
import com.usky.common.mybatis.core.AbstractCrudService;
import com.usky.oa.service.OaJbdDocumentService;
import com.usky.oa.service.OaQjdDocumentService;
import com.usky.oa.service.utils.OaSendMessageCenter;
import org.apache.commons.lang3.RandomUtils;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.time.LocalDateTime;
import java.time.format.DateTimeFormatter;
import java.util.Collections;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * <p>
 * 单据总表 服务实现类
 * </p>
 *
 * @author fu
 * @since 2024-12-26
 */
@Service
public class OaDocumentServiceImpl extends AbstractCrudService<OaDocumentMapper, OaDocument> implements OaDocumentService {

    @Autowired
    private OaDocumentMapper oaDocumentMapper;

    @Autowired
    private OaQjdDocumentMapper oaQjdDocumentMapper;

    @Autowired
    private OaQjdDocumentService oaQjdDocumentService;

    @Autowired
    private OaJbdDocumentService oaJbdDocumentService;

    @Autowired
    private OaJbdDocumentMapper oaJbdDocumentMapper;

    @Autowired
    private OaDocumentMapper documentMapper;

    @Autowired
    private OaFormDefinitionMapper oaFormDefinitionMapper;

    @Autowired
    private OaApprovalMapper oaApprovalMapper;

    @Autowired
    private OaSendMessageCenter oaSendMessageCenter;

    private static final String TABLE_HEAD = "oa_";
    private static final String TABLE_SUFFIX = "_document";

    // 我的申请
    @Override
    public CommonPage<OaDocument> myApplication(String docNo, String type, Integer status, String startTime, String endTime, Integer pageNum, Integer pageSize) {
        Long userId = SecurityUtils.getUserId();
        Integer tenantId = SecurityUtils.getTenantId();

        LambdaQueryWrapper<OaDocument> wrapper = Wrappers.lambdaQuery();
        wrapper.eq(OaDocument::getTenantId, tenantId).eq(OaDocument::getProposer, userId)
                .like(StringUtils.isNotBlank(docNo), OaDocument::getDocNo, docNo)
                .eq(StringUtils.isNotBlank(type), OaDocument::getType, type)
                .eq(status != null, OaDocument::getDocStatus, status)
                .between(StringUtils.isNotBlank(startTime) && StringUtils.isNotBlank(endTime), OaDocument::getCreateTime, startTime, endTime)
                .orderByDesc(OaDocument::getId);

        return ToCommonPage(page(new Page<>(pageNum, pageSize), wrapper));
    }

    // 单据详情
    @Override
    public Object documentDetails(String docNo) {
        Object documentDetails = null;
        String type = getFormSign(docNo);

        switch (type) {
            case "QJD":
                documentDetails = oaQjdDocumentMapper.selectOne(Wrappers.lambdaQuery(OaQjdDocument.class).eq(OaQjdDocument::getDocNo, docNo));
                break;
            case "JBD":
                documentDetails = oaJbdDocumentMapper.selectOne(Wrappers.lambdaQuery(OaJbdDocument.class).eq(OaJbdDocument::getDocNo, docNo));
                break;
        }

        return documentDetails;
    }

    // 删除单据
    @Override
    public void delDocument(String docNo) {
        String type = getFormSign(docNo);
        switch (type) {
            case "QJD":
                oaQjdDocumentMapper.delete(Wrappers.lambdaQuery(OaQjdDocument.class).eq(OaQjdDocument::getDocNo, docNo));
                break;
            case "JBD":
                oaJbdDocumentMapper.delete(Wrappers.lambdaQuery(OaJbdDocument.class).eq(OaJbdDocument::getDocNo, docNo));
                break;
        }
        oaDocumentMapper.delete(Wrappers.lambdaQuery(OaDocument.class).eq(OaDocument::getDocNo, docNo));
    }

    // 新增单据
    @Override
    public void addDoc(JSONObject oaDocument) {
        LambdaQueryWrapper<OaFormDefinition> queryWrapper = Wrappers.lambdaQuery();
        queryWrapper.select(OaFormDefinition::getFieldInfo).eq(OaFormDefinition::getId, oaDocument.get("formId"));
        OaFormDefinition oaFormDefinition = oaFormDefinitionMapper.selectOne(queryWrapper);
        String sign = oaFormDefinition.getFieldInfo();

        switch (sign) {
            case "QJD":
                oaQjdDocumentService.addQjDocument(JSON.parseObject(oaDocument.toJSONString(), OaQjdDocument.class));
                break;
            case "JBD":
                oaJbdDocumentService.add(JSON.parseObject(oaDocument.toJSONString(), OaJbdDocument.class));
                break;
        }
    }

    // 更新单据
    @Override
    public void updateDoc(JSONObject jsonDocument) {
        LambdaQueryWrapper<OaFormDefinition> queryWrapper = Wrappers.lambdaQuery();
        queryWrapper.select(OaFormDefinition::getFieldInfo).eq(OaFormDefinition::getId, jsonDocument.get("formId"));
        OaFormDefinition oaFormDefinition = oaFormDefinitionMapper.selectOne(queryWrapper);
        String sign = oaFormDefinition.getFieldInfo();

        switch (sign) {
            case "QJD":
                oaQjdDocumentService.updateQjDocument(JSON.parseObject(jsonDocument.toJSONString(), OaQjdDocument.class));
                break;
            case "JBD":
                oaJbdDocumentService.update(JSON.parseObject(jsonDocument.toJSONString(), OaJbdDocument.class));
                break;
        }
    }

    // 提交单据
    @Override
    public void submit(String docNo, Integer docStatus) {
        if (docStatus != 1) {
            throw new RuntimeException("单据提交异常，请联系管理员！");
        } else {
            LambdaQueryWrapper<OaDocument> queryWrapper = Wrappers.lambdaQuery();
            queryWrapper.eq(OaDocument::getDocNo, docNo);
            OaDocument oaDocument = oaDocumentMapper.selectOne(queryWrapper);
            if (oaDocument.getDocStatus() == 1) {
                throw new RuntimeException("单据已提交，请勿重复提交！");
            }
        }

        Long deptId = SecurityUtils.getLoginUser().getSysUser().getDeptId();
        Integer tenantId = SecurityUtils.getTenantId();
        String username = SecurityUtils.getUsername();
        Long userId = SecurityUtils.getUserId();
        LocalDateTime now = LocalDateTime.now();

        String formSign = getFormSign(docNo);

        switch (formSign) {
            case "QJD":
                oaQjdDocumentMapper.update(null, Wrappers.lambdaUpdate(OaQjdDocument.class)
                        .set(OaQjdDocument::getDocStatus, docStatus)
                        .set(OaQjdDocument::getUpdateBy, username)
                        .set(OaQjdDocument::getUpdateTime, now)
                        .eq(OaQjdDocument::getDocNo, docNo));
                break;
            case "JBD":
                oaJbdDocumentMapper.update(null, Wrappers.lambdaUpdate(OaJbdDocument.class)
                        .set(OaJbdDocument::getDocStatus, docStatus)
                        .set(OaJbdDocument::getUpdateBy, username)
                        .set(OaJbdDocument::getUpdateTime, now)
                        .eq(OaJbdDocument::getDocNo, docNo));
                break;
        }
        oaDocumentMapper.update(null, Wrappers.lambdaUpdate(OaDocument.class)
                .set(OaDocument::getDocStatus, docStatus)
                .set(OaDocument::getUpdateBy, username)
                .set(OaDocument::getUpdateTime, now)
                .eq(OaDocument::getDocNo, docNo));

        // 生成审批记录
        OaApproval oaApproval = new OaApproval();
        oaApproval.setDocNo(docNo);
        oaApproval.setType(0);
        oaApproval.setApprovalStatus(1);
        // 随机选择一个审批人（涛、哲)
        long approvalUid = RandomUtils.nextBoolean() ? 101 : 105;
        oaApproval.setApprovalUid(approvalUid);
        oaApproval.setProposer(Math.toIntExact(userId));
        oaApproval.setCreateBy(username);
        oaApproval.setCreateTime(now);
        oaApproval.setSubmitDate(now);
        oaApproval.setDeptId(deptId);
        oaApproval.setTenantId(tenantId);
        oaApprovalMapper.insert(oaApproval);

        // 发送审批消息
        List<Long> receivers = Collections.singletonList(oaApproval.getApprovalUid());
        oaSendMessageCenter.sendAsyncMessage(oaApproval.getCreateBy(), userId, oaApproval.getId(), receivers,3);
    }

    // 获取单据类型
    private String getFormSign(String docNo) {
        if (StringUtils.isBlank(docNo)) {
            throw new RuntimeException("单据号不能为空！");
        } else if (docNo.split("-").length != 2 ||
                docNo.split("-")[0].length() > 4 ||
                docNo.split("-")[0].length() < 3 ||
                docNo.split("-")[1].length() != 12) {
            throw new RuntimeException("单据号格式错误！");
        }
        return docNo.split("-")[0];
    }

}
